#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
module Integrations
  class BaseStrategy
    def execute(action, params)
      raise NotImplementedError, "This method should be implemented by subclasses"
    end

    def actions
      raise NotImplementedError, "This method should be implemented by subclasses"
    end

    def self.strategy_name
      const_get(:NAME)
    end

    def self.action_params(action)
      const_get(:AVAILABLE_ACTIONS)[action]
    end
  end

  class OtrsStrategy < BaseStrategy
    # ----------------------------------------------
    # Create ticket API Documentation: https://otrscommunityedition.com/doc/api/otrs/6.0/Perl/Kernel/GenericInterface/Operation/Ticket/TicketCreate.pm.html
    # ----------------------------------------------

    include HTTParty

    # ==========================================================
    # Constants
    # ==========================================================
    NAME = "otrs"
    AVAILABLE_ACTIONS = {
      "create_ticket" => {
        ticket_title: { required: true, type: :string },
        customer_user: { required: true, type: :string },
        queue_id: { required: false, type: :integer },
        priority_id: { required: false, type: :integer },
        state_id: { required: false, type: :integer },
        article_subject: { required: true, type: :string },
        article_body: { required: true, type: :string },
      },
    }

    def execute(action, params)
      if self.class.action_params(action)
        send(action, params)
      else
        { error: "Unknown action: #{action}" }
      end
    end

    def actions
      self.class.const_get(:AVAILABLE_ACTIONS).keys
    end

    private

    # ----------------------------------------------
    # POST Method to create OTRS tickets
    # ----------------------------------------------
    # == Parameters:
    # ticket_title::
    #   String with the title of the ticket.
    # queue_id::
    #   (Optional) Integer with the queue ID (default: 3).
    # priority_id::
    #   (Optional) Integer with the priority ID (default: 3).
    # state_id::
    #   (Optional) Integer with the state ID of the ticket (default: 1).
    # costumer_user::
    #   String with the user associated of the ticket.
    # article_subject::
    #   String with the subject of the article.
    # article_body::
    #   String with the body of the article.
    #
    # == Returns:
    # The response from the OTRS API.
    def create_ticket(params)
      key_post = Integration.find_by(name: "otrs_key_post")
      web_service = Integration.find_by(name: "otrs_web_service")
      create_ticket_route_mapping = Integration.find_by(name: "create_ticket_route_mapping")
      username = Integration.find_by(name: "otrs_username")
      password = Integration.find_by(name: "otrs_password")

      unless key_post && web_service && username && password && create_ticket_route_mapping
        return { error: "OTRS create ticket API endpoint not found." }
      end

      base_url = "#{key_post.value}/otrs/nph-genericinterface.pl/Webservice/#{web_service.value}/#{create_ticket_route_mapping.value}?UserLogin=#{username.value}&Password=#{password.value}"

      ticket_params = {
        Ticket: {
          Title: params[:ticket_title],
          QueueID: params[:queue_id] || 3,
          PriorityID: params[:priority_id] || 3,
          StateID: params[:state_id] || 1,
          CustomerUser: params[:customer_user],
        },
        Article: {
          Subject: params[:article_subject],
          Body: params[:article_body],
          ContentType: "text/plain; charset=utf8",
        },
      }

      response = nil

      begin
        response = HTTParty.post(
          base_url,
          headers: { "Content-Type" => "application/json" },
          body: ticket_params.to_json,
        )
      rescue StandardError => e
        return { error: "HTTP request failed", message: e.message }
      end

      response.parsed_response
    end
  end
end
